@@ -21,7 +21,7 @@
 #define GCONF_XML_DIR_H
 
 #include <gconf/gconf.h>
-#include <gnome-xml/tree.h>
+#include <libxml/tree.h>
 
 /* Dir stores the information about a given directory */
 
@@ -45,7 +45,7 @@ const gchar*   dir_get_name        (Dir          *d);
 /* key should have no slashes in it */
 void           dir_set_value       (Dir          *d,
                                     const gchar  *relative_key,
-                                    GConfValue   *value,
+                                    const GConfValue *value,
                                     GError  **err);
 GConfValue*    dir_get_value       (Dir          *d,
                                     const gchar  *relative_key,
