@@ -32,7 +32,6 @@
 #include <sys/wait.h>
 #include <sys/time.h>
 #include <unistd.h>
-#include <bonobo-activation/bonobo-activation.h>
 
 /* Returns TRUE if there was an error, frees exception, sets err */
 static gboolean gconf_handle_corba_exception(CORBA_Environment* ev, GError** err);
@@ -79,6 +78,9 @@ struct _GConfEngine {
 
   gpointer user_data;
   GDestroyNotify dnotify;
+
+  gpointer owner;
+  int owner_use_count;
   
   guint is_default : 1;
 
@@ -130,6 +132,11 @@ static ConfigDatabase gconf_engine_get_database (GConfEngine     *conf,
                                                  GError         **err);
 
 
+#define CHECK_OWNER_USE(engine)   \
+  do { if ((engine)->owner && (engine)->owner_use_count == 0) \
+     g_warning ("You can't use a GConfEngine that has an active GConfClient wrapper object. Use GConfClient API instead.");  \
+  } while (0)
+
 static void         register_engine           (GConfEngine    *conf);
 static void         unregister_engine         (GConfEngine    *conf);
 static GConfEngine *lookup_engine             (const gchar    *address);
@@ -153,8 +160,6 @@ static void       ctable_insert              (CnxnTable           *ct,
                                               GConfCnxn           *cnxn);
 static void       ctable_remove              (CnxnTable           *ct,
                                               GConfCnxn           *cnxn);
-static void       ctable_remove_by_client_id (CnxnTable           *ct,
-                                              guint                client_id);
 static GSList*    ctable_remove_by_conf      (CnxnTable           *ct,
                                               GConfEngine         *conf);
 static GConfCnxn* ctable_lookup_by_client_id (CnxnTable           *ct,
@@ -176,6 +181,9 @@ gconf_engine_blank (gboolean remote)
 
   conf->refcount = 1;
   
+  conf->owner = NULL;
+  conf->owner_use_count = 0;
+  
   if (remote)
     {
       conf->database = CORBA_OBJECT_NIL;
@@ -192,10 +200,38 @@ gconf_engine_blank (gboolean remote)
       conf->is_local = TRUE;
       conf->is_default = FALSE;
     }
-    
+  
   return conf;
 }
 
+void
+gconf_engine_set_owner (GConfEngine *engine,
+                        gpointer     client)
+{
+  g_return_if_fail (engine->owner_use_count == 0);
+  
+  engine->owner = client;
+}
+
+void
+gconf_engine_push_owner_usage (GConfEngine *engine,
+                               gpointer     client)
+{
+  g_return_if_fail (engine->owner == client);
+
+  engine->owner_use_count += 1;
+}
+
+void
+gconf_engine_pop_owner_usage  (GConfEngine *engine,
+                               gpointer     client)
+{
+  g_return_if_fail (engine->owner == client);
+  g_return_if_fail (engine->owner_use_count > 0);
+
+  engine->owner_use_count -= 1;
+}
+
 static GHashTable *engines_by_db = NULL;
 
 static GConfEngine *
@@ -287,7 +323,7 @@ gconf_engine_connect (GConfEngine *conf,
       if (err)
         *err = gconf_error_new(GCONF_ERROR_BAD_ADDRESS,
                                _("Server couldn't resolve the address `%s'"),
-                               conf->address);
+                               conf->address ? conf->address : "default");
           
       return FALSE;
     }
@@ -397,10 +433,11 @@ gconf_engine_get_default (void)
 
       default_engine = conf;
       
-      /* Ignore errors, we never return a NULL
-       * default database
+      /* Ignore errors, we never return a NULL default database, and
+       * since we aren't starting if it isn't found, we'll probably
+       * get errors half the time anyway.
        */
-      gconf_engine_connect (conf, TRUE, NULL);
+      gconf_engine_connect (conf, FALSE, NULL);
     }
   else
     conf->refcount += 1;
@@ -564,8 +601,13 @@ gconf_engine_notify_add(GConfEngine* conf,
   CORBA_Environment ev;
   GConfCnxn* cnxn;
   gint tries = 0;
-
+  ConfigDatabase3_PropList properties;
+#define NUM_PROPERTIES 1
+  ConfigStringProperty properties_buffer[1];
+  
   g_return_val_if_fail(!gconf_engine_is_local(conf), 0);
+
+  CHECK_OWNER_USE (conf);
   
   if (gconf_engine_is_local(conf))
     {
@@ -575,6 +617,16 @@ gconf_engine_notify_add(GConfEngine* conf,
 
       return 0;
     }
+
+  properties._buffer = properties_buffer;
+  properties._length = NUM_PROPERTIES;
+  properties._maximum = NUM_PROPERTIES;
+  properties._release = CORBA_FALSE; /* don't free static buffer */
+
+  properties._buffer[0].key = "name";
+  properties._buffer[0].value = g_get_prgname ();
+  if (properties._buffer[0].value == NULL)
+    properties._buffer[0].value = "unknown";
   
   CORBA_exception_init(&ev);
 
@@ -589,10 +641,24 @@ gconf_engine_notify_add(GConfEngine* conf,
   
   /* Should have aborted the program in this case probably */
   g_return_val_if_fail(cl != CORBA_OBJECT_NIL, 0);
-
-  id = ConfigDatabase_add_listener(db,
-                                   (gchar*)namespace_section, 
-                                   cl, &ev);
+  
+  id = ConfigDatabase3_add_listener_with_properties (db,
+                                                     (gchar*)namespace_section, 
+                                                     cl,
+                                                     &properties,
+                                                     &ev);
+  
+  if (ev._major == CORBA_SYSTEM_EXCEPTION &&
+      CORBA_exception_id (&ev) &&
+      strcmp (CORBA_exception_id (&ev), "IDL:CORBA/BAD_OPERATION:1.0") == 0)
+    {
+      CORBA_exception_free (&ev);
+      CORBA_exception_init (&ev);      
+  
+      id = ConfigDatabase_add_listener(db,
+                                       (gchar*)namespace_section, 
+                                       cl, &ev);
+    }
   
   if (gconf_server_broken(&ev))
     {
@@ -617,13 +683,15 @@ gconf_engine_notify_add(GConfEngine* conf,
 
 void         
 gconf_engine_notify_remove(GConfEngine* conf,
-                    guint client_id)
+                           guint client_id)
 {
   GConfCnxn* gcnxn;
   CORBA_Environment ev;
   ConfigDatabase db;
   gint tries = 0;
 
+  CHECK_OWNER_USE (conf);
+  
   if (gconf_engine_is_local(conf))
     return;
   
@@ -691,6 +759,8 @@ gconf_engine_get_fuller (GConfEngine *conf,
   g_return_val_if_fail(key != NULL, NULL);
   g_return_val_if_fail(err == NULL || *err == NULL, NULL);
 
+  CHECK_OWNER_USE (conf);
+  
   if (!gconf_key_check(key, err))
     return NULL;
 
@@ -848,6 +918,8 @@ gconf_engine_get_entry(GConfEngine* conf,
   GConfEntry *entry;
   gchar *schema_name;
 
+  CHECK_OWNER_USE (conf);
+  
   schema_name = NULL;
   error = NULL;
   val = gconf_engine_get_fuller (conf, key, locale, use_schema_default,
@@ -905,6 +977,8 @@ gconf_engine_get_default_from_schema (GConfEngine* conf,
   g_return_val_if_fail(conf != NULL, NULL);
   g_return_val_if_fail(key != NULL, NULL);
   g_return_val_if_fail(err == NULL || *err == NULL, NULL);
+
+  CHECK_OWNER_USE (conf);
   
   if (!gconf_key_check(key, err))
     return NULL;
@@ -990,10 +1064,15 @@ gconf_engine_set (GConfEngine* conf, const gchar* key,
   g_return_val_if_fail( (value->type != GCONF_VALUE_LIST) ||
                         (gconf_value_get_list_type(value) != GCONF_VALUE_INVALID), FALSE);
   g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
+
+  CHECK_OWNER_USE (conf);
   
   if (!gconf_key_check(key, err))
     return FALSE;
 
+  if (!gconf_value_validate (value, err))
+    return FALSE;
+  
   if (gconf_engine_is_local(conf))
     {
       GError* error = NULL;
@@ -1057,15 +1136,17 @@ gconf_engine_set (GConfEngine* conf, const gchar* key,
 }
 
 gboolean
-gconf_engine_unset(GConfEngine* conf, const gchar* key, GError** err)
+gconf_engine_unset (GConfEngine* conf, const gchar* key, GError** err)
 {
   CORBA_Environment ev;
   ConfigDatabase db;
   gint tries = 0;
 
-  g_return_val_if_fail(conf != NULL, FALSE);
-  g_return_val_if_fail(key != NULL, FALSE);
-  g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
+  g_return_val_if_fail (conf != NULL, FALSE);
+  g_return_val_if_fail (key != NULL, FALSE);
+  g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
+
+  CHECK_OWNER_USE (conf);
   
   if (!gconf_key_check(key, err))
     return FALSE;
@@ -1105,11 +1186,11 @@ gconf_engine_unset(GConfEngine* conf, const gchar* key, GError** err)
       return FALSE;
     }
 
-  ConfigDatabase_unset(db,
-                     (gchar*)key,
-                     &ev);
+  ConfigDatabase_unset (db,
+                        (gchar*)key,
+                        &ev);
 
-  if (gconf_server_broken(&ev))
+  if (gconf_server_broken (&ev))
     {
       if (tries < MAX_RETRIES)
         {
@@ -1120,10 +1201,10 @@ gconf_engine_unset(GConfEngine* conf, const gchar* key, GError** err)
         }
     }
   
-  if (gconf_handle_corba_exception(&ev, err))
+  if (gconf_handle_corba_exception (&ev, err))
     return FALSE;
 
-  g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
+  g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
   
   return TRUE;
 }
@@ -1140,6 +1221,8 @@ gconf_engine_associate_schema  (GConfEngine* conf, const gchar* key,
   g_return_val_if_fail(key != NULL, FALSE);
   g_return_val_if_fail(schema_key != NULL, FALSE);
   g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
+
+  CHECK_OWNER_USE (conf);
   
   if (!gconf_key_check(key, err))
     return FALSE;
@@ -1242,6 +1325,8 @@ gconf_engine_all_entries(GConfEngine* conf, const gchar* dir, GError** err)
   g_return_val_if_fail(conf != NULL, NULL);
   g_return_val_if_fail(dir != NULL, NULL);
   g_return_val_if_fail(err == NULL || *err == NULL, NULL);
+
+  CHECK_OWNER_USE (conf);
   
   if (!gconf_key_check(dir, err))
     return NULL;
@@ -1389,6 +1474,8 @@ gconf_engine_all_dirs(GConfEngine* conf, const gchar* dir, GError** err)
   g_return_val_if_fail(conf != NULL, NULL);
   g_return_val_if_fail(dir != NULL, NULL);
   g_return_val_if_fail(err == NULL || *err == NULL, NULL);
+
+  CHECK_OWNER_USE (conf);
   
   if (!gconf_key_check(dir, err))
     return NULL;
@@ -1483,6 +1570,8 @@ gconf_engine_suggest_sync(GConfEngine* conf, GError** err)
   g_return_if_fail(conf != NULL);
   g_return_if_fail(err == NULL || *err == NULL);
 
+  CHECK_OWNER_USE (conf);
+  
   if (gconf_engine_is_local(conf))
     {
       GError* error = NULL;
@@ -1546,6 +1635,11 @@ gconf_clear_cache(GConfEngine* conf, GError** err)
   g_return_if_fail(conf != NULL);
   g_return_if_fail(err == NULL || *err == NULL);
 
+  /* don't disallow non-owner use here since you can't do this
+   * via GConfClient API and calling this function won't break
+   * GConfClient anyway
+   */
+  
   if (gconf_engine_is_local(conf))
     {
       GError* error = NULL;
@@ -1671,6 +1765,8 @@ gconf_engine_dir_exists(GConfEngine *conf, const gchar *dir, GError** err)
   g_return_val_if_fail(conf != NULL, FALSE);
   g_return_val_if_fail(dir != NULL, FALSE);
   g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
+
+  CHECK_OWNER_USE (conf);
   
   if (!gconf_key_check(dir, err))
     return FALSE;
@@ -1730,6 +1826,8 @@ gconf_engine_remove_dir (GConfEngine* conf,
   g_return_if_fail(conf != NULL);
   g_return_if_fail(dir != NULL);
   g_return_if_fail(err == NULL || *err == NULL);
+
+  CHECK_OWNER_USE (conf);
   
   if (!gconf_key_check(dir, err))
     return;
@@ -1777,6 +1875,8 @@ gconf_engine_key_is_writable  (GConfEngine *conf,
   gboolean is_writable = TRUE;
   GConfValue *val;
 
+  CHECK_OWNER_USE (conf);
+  
   /* FIXME implement IDL to allow getting only writability
    * (not that urgent since GConfClient caches this crap
    * anyway)
@@ -1842,20 +1942,18 @@ static ConfigServer   server = CORBA_OBJECT_NIL;
 
 /* errors in here should be GCONF_ERROR_NO_SERVER */
 static ConfigServer
-try_to_contact_server(gboolean start_if_not_found, GError** err)
+try_to_contact_server (gboolean start_if_not_found,
+                       GError **err)
 {
   CORBA_Environment ev;
-  Bonobo_ActivationFlags flags;
   
-  CORBA_exception_init(&ev);
+  /* Try to launch server */      
+  server = gconf_activate_server (start_if_not_found,
+                                  err);
+    
+  /* Try to ping server, by adding ourselves as a client */
+  CORBA_exception_init (&ev);   
 
-  flags = 0;
-  if (!start_if_not_found)
-    flags |= Bonobo_ACTIVATION_FLAG_EXISTING_ONLY;
-  
-  server = bonobo_activation_activate_from_id ("OAFAID:["IID"]", flags, NULL, &ev);
-
-  /* So try to ping server, by adding ourselves as a client */
   if (!CORBA_Object_is_nil (server, &ev))
     {
       ConfigServer_add_client (server,
@@ -1865,33 +1963,19 @@ try_to_contact_server(gboolean start_if_not_found, GError** err)
       if (ev._major != CORBA_NO_EXCEPTION)
 	{
 	  server = CORBA_OBJECT_NIL;
-	  if (err)
-	    *err = gconf_error_new(GCONF_ERROR_NO_SERVER,
-                                   _("Adding client to server's list failed, CORBA error: %s"),
-				   CORBA_exception_id(&ev));
+          g_set_error (err,
+                       GCONF_ERROR,
+                       GCONF_ERROR_NO_SERVER,
+                       _("Adding client to server's list failed, CORBA error: %s"),
+                       CORBA_exception_id (&ev));
 
           CORBA_exception_free(&ev);
 	}
     }
-  else
-    {
-      if (gconf_handle_oaf_exception(&ev, err))
-        {
-          /* Make the errno more specific */
-          if (err && *err)
-            (*err)->code = GCONF_ERROR_NO_SERVER;
-        }
-
-      if (err && *err == NULL)
-        *err = gconf_error_new(GCONF_ERROR_NO_SERVER,
-                               _("Error contacting configuration server: bonobo-activation returned nil from bonobo_activation_activate_from_id() and did not set an exception explaining the problem. This is a bug in the bonobo-activation package; something went wrong in bonobo-activation, and no error was reported. This is not a bug in the GConf package. Do not report a GConf bug unless you have information indicating what went wrong with bonobo-activation that was caused by GConf."));
-    }
 
 #ifdef GCONF_ENABLE_DEBUG      
   if (server == CORBA_OBJECT_NIL && start_if_not_found)
-    {
-      g_return_val_if_fail(err == NULL || *err != NULL, server);
-    }
+    g_return_val_if_fail (err == NULL || *err != NULL, server);
 #endif
   
   return server;
@@ -2113,32 +2197,21 @@ drop_all_caches (PortableServer_Servant     _servant,
 static ConfigListener 
 gconf_get_config_listener(void)
 {
-  return listener;
-}
-
-static gboolean have_initted = FALSE;
-
-void
-gconf_preinit(gpointer app, gpointer mod_info)
-{
-}
-
-void
-gconf_postinit(gpointer app, gpointer mod_info)
-{
+  static ConfigListener listener = CORBA_OBJECT_NIL;
+  
   if (listener == CORBA_OBJECT_NIL)
     {
       CORBA_Environment ev;
-      PortableServer_ObjectId* objid;
       PortableServer_POA poa;
 
-      CORBA_exception_init(&ev);
-      POA_ConfigListener__init(&poa_listener_servant, &ev);
+      CORBA_exception_init (&ev);
+      POA_ConfigListener__init (&poa_listener_servant, &ev);
       
       g_assert (ev._major == CORBA_NO_EXCEPTION);
 
-      poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(
-	      bonobo_activation_orb_get(), "RootPOA", &ev);
+      poa =
+        (PortableServer_POA) CORBA_ORB_resolve_initial_references (gconf_orb_get (),
+                                                                   "RootPOA", &ev);
 
       g_assert (ev._major == CORBA_NO_EXCEPTION);
 
@@ -2146,10 +2219,6 @@ gconf_postinit(gpointer app, gpointer mod_info)
 
       g_assert (ev._major == CORBA_NO_EXCEPTION);
 
-      objid = PortableServer_POA_activate_object(poa, &poa_listener_servant, &ev);
-
-      g_assert (ev._major == CORBA_NO_EXCEPTION);
-      
       listener = PortableServer_POA_servant_to_reference(poa,
                                                          &poa_listener_servant,
                                                          &ev);
@@ -2157,49 +2226,33 @@ gconf_postinit(gpointer app, gpointer mod_info)
       g_assert (listener != CORBA_OBJECT_NIL);
       g_assert (ev._major == CORBA_NO_EXCEPTION);
     }
-
-  have_initted = TRUE;
+  
+  return listener;
+}
+     
+void
+gconf_preinit (gpointer app, gpointer mod_info)
+{
+  /* Deprecated */
 }
 
+void
+gconf_postinit (gpointer app, gpointer mod_info)
+{
+  /* Deprecated */
+}
+
+/* All deprecated */
 const char gconf_version[] = VERSION;
 
 struct poptOption gconf_options[] = {
   {NULL}
 };
 
+/* Also deprecated */
 gboolean     
-gconf_init           (int argc, char **argv, GError** err)
+gconf_init (int argc, char **argv, GError** err)
 {
-  CORBA_ORB orb = CORBA_OBJECT_NIL;
-
-  if (have_initted)
-    {
-      g_warning("Attempt to init GConf a second time");
-      return FALSE;
-    }
-
-  gconf_preinit(NULL, NULL);
-
-  if (!bonobo_activation_is_initialized())
-    {
-      orb = bonobo_activation_init(argc, argv);
-    }
-  else
-    {
-      orb = bonobo_activation_orb_get();
-    }
-      
-  gconf_postinit(NULL, NULL);
-
-  if(!have_initted)
-    {
-      if (err == NULL)
-        {
-          fprintf(stderr, _("Failed to init GConf, exiting\n"));
-          exit (1);
-        }
-      return FALSE;
-    }
   
   return TRUE;
 }
@@ -2207,7 +2260,7 @@ gconf_init           (int argc, char **argv, GError** err)
 gboolean
 gconf_is_initialized (void)
 {
-  return have_initted;
+  return TRUE;
 }
 
 /* 
@@ -2220,6 +2273,8 @@ gconf_is_initialized (void)
 /* Key/dir validity is exactly the same, except that '/' must be a dir, 
    but we are sort of ignoring that for now. */
 
+/* Also, keys can contain only ASCII */
+
 static const gchar invalid_chars[] = " \t\r\n\"$&<>,+=#!()'|{}[]?~`;%\\";
 
 gboolean     
@@ -2271,7 +2326,15 @@ gconf_valid_key      (const gchar* key, gchar** why_invalid)
           const gchar* inv = invalid_chars;
 
           just_saw_slash = FALSE;
-
+          
+          if (((unsigned char)*s) > 127)
+            {
+              if (why_invalid != NULL)
+                *why_invalid = g_strdup_printf (_("'%c' is not an ASCII character, so isn't allowed in key names"),
+                                                *s);
+              return FALSE;
+            }
+          
           while (*inv)
             {
               if (*inv == *s)
@@ -2298,6 +2361,122 @@ gconf_valid_key      (const gchar* key, gchar** why_invalid)
     return TRUE;
 }
 
+/**
+ * gconf_escape_key:
+ * @arbitrary_text: some text in any encoding or format
+ * @len: length of @arbitrary_text in bytes, or -1 if @arbitrary_text is nul-terminated
+ * 
+ * Escape @arbitrary_text such that it's a valid key element (i.e. one
+ * part of the key path). The escaped key won't pass gconf_valid_key()
+ * because it isn't a whole key (i.e. it doesn't have a preceding
+ * slash), but prepending a slash to the escaped text should always
+ * result in a valid key.
+ * 
+ * Return value: a nul-terminated valid GConf key
+ **/
+char*
+gconf_escape_key (const char *arbitrary_text,
+                  int         len)
+{
+  const char *p;
+  const char *end;
+  GString *retval;
+
+  g_return_val_if_fail (arbitrary_text != NULL, NULL);
+  
+  /* Nearly all characters we would normally use for escaping aren't allowed in key
+   * names, so we use @ for that.
+   *
+   * Invalid chars and @ itself are escaped as @xxx@ where xxx is the
+   * Latin-1 value in decimal
+   */
+
+  if (len < 0)
+    len = strlen (arbitrary_text);
+
+  retval = g_string_new ("");
+
+  p = arbitrary_text;
+  end = arbitrary_text + len;
+  while (p != end)
+    {
+      if (*p == '/' || *p == '.' || *p == '@' || ((guchar) *p) > 127 ||
+          strchr (invalid_chars, *p))
+        {
+          g_string_append_c (retval, '@');
+          g_string_append_printf (retval, "%u", (unsigned int) *p);
+          g_string_append_c (retval, '@');
+        }
+      else
+        g_string_append_c (retval, *p);
+      
+      ++p;
+    }
+
+  return g_string_free (retval, FALSE);
+}
+
+/**
+ * gconf_unescape_key:
+ * @escaped_key: a key created with gconf_escape_key()
+ * @len: length of @escaped_key in bytes, or -1 if @escaped_key is nul-terminated
+ * 
+ * Converts a string escaped with gconf_escape_key() back into its original
+ * form.
+ * 
+ * Return value: the original string that was escaped to create @escaped_key
+ **/
+char*
+gconf_unescape_key (const char *escaped_key,
+                    int         len)
+{
+  const char *p;
+  const char *end;
+  const char *start_seq;
+  GString *retval;
+
+  g_return_val_if_fail (escaped_key != NULL, NULL);
+  
+  if (len < 0)
+    len = strlen (escaped_key);
+
+  retval = g_string_new ("");
+
+  p = escaped_key;
+  end = escaped_key + len;
+  start_seq = NULL;
+  while (p != end)
+    {
+      if (start_seq)
+        {
+          if (*p == '@')
+            {
+              /* *p is the @ that ends a seq */
+              char *end;
+              guchar val;
+              
+              val = strtoul (start_seq, &end, 10);
+              if (start_seq != end)
+                g_string_append_c (retval, val);
+              
+              start_seq = NULL;
+            }
+        }
+      else
+        {
+          if (*p == '@')
+            start_seq = p + 1;
+          else
+            g_string_append_c (retval, *p);
+        }
+
+      ++p;
+    }
+
+  return g_string_free (retval, FALSE);
+}
+
+
 gboolean
 gconf_key_is_below   (const gchar* above, const gchar* below)
 {
@@ -2424,18 +2603,6 @@ ctable_remove(CnxnTable* ct, GConfCnxn* cnxn)
   g_hash_table_remove (ct->client_ids, &cnxn->client_id);
 }
 
-static void       
-ctable_remove_by_client_id(CnxnTable* ct, guint client_id)
-{
-  GConfCnxn* cnxn;
-
-  cnxn = ctable_lookup_by_client_id (ct, client_id);
-
-  g_return_if_fail (cnxn != NULL);
-
-  ctable_remove (ct, cnxn);
-}
-
 struct RemoveData {
   GSList* removed;
   GConfEngine* conf;
@@ -2524,28 +2691,35 @@ ctable_reinstall (CnxnTable* ct,
  */
 
 void          
-gconf_shutdown_daemon(GError** err)
+gconf_shutdown_daemon (GError** err)
 {
   CORBA_Environment ev;
   ConfigServer cs;
 
-  cs = gconf_get_config_server(FALSE, err); /* Don't want to spawn it if it's already down */
+  cs = gconf_get_config_server (FALSE, err); /* Don't want to spawn it if it's already down */
 
-  if (cs == CORBA_OBJECT_NIL)
+  if (err && *err && (*err)->code == GCONF_ERROR_NO_SERVER)
     {
-
+      /* No server is hardly an error here */
+      g_error_free (*err);
+      *err = NULL;
+    }
+  
+  if (cs == CORBA_OBJECT_NIL)
+    {      
+      
       return;
     }
 
-  CORBA_exception_init(&ev);
+  CORBA_exception_init (&ev);
 
-  ConfigServer_shutdown(cs, &ev);
+  ConfigServer_shutdown (cs, &ev);
 
   if (ev._major != CORBA_NO_EXCEPTION)
     {
       if (err)
-        *err = gconf_error_new(GCONF_ERROR_FAILED, _("Failure shutting down config server: %s"),
-                               CORBA_exception_id(&ev));
+        *err = gconf_error_new (GCONF_ERROR_FAILED, _("Failure shutting down config server: %s"),
+                                CORBA_exception_id (&ev));
 
       CORBA_exception_free(&ev);
     }
@@ -2764,7 +2938,7 @@ gconf_engine_get_schema  (GConfEngine* conf, const gchar* key, GError** err)
 
 GSList*
 gconf_engine_get_list    (GConfEngine* conf, const gchar* key,
-                   GConfValueType list_type, GError** err)
+                          GConfValueType list_type, GError** err)
 {
   GConfValue* val;
 
@@ -2863,7 +3037,7 @@ error_checked_set(GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_float   (GConfEngine* conf, const gchar* key,
-                   gdouble val, GError** err)
+                          gdouble val, GError** err)
 {
   GConfValue* gval;
 
@@ -2880,7 +3054,7 @@ gconf_engine_set_float   (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_int     (GConfEngine* conf, const gchar* key,
-                   gint val, GError** err)
+                          gint val, GError** err)
 {
   GConfValue* gval;
 
@@ -2897,14 +3071,16 @@ gconf_engine_set_int     (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_string  (GConfEngine* conf, const gchar* key,
-                    const gchar* val, GError** err)
+                          const gchar* val, GError** err)
 {
   GConfValue* gval;
 
-  g_return_val_if_fail(val != NULL, FALSE);
-  g_return_val_if_fail(conf != NULL, FALSE);
-  g_return_val_if_fail(key != NULL, FALSE);
-  g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
+  g_return_val_if_fail (val != NULL, FALSE);
+  g_return_val_if_fail (conf != NULL, FALSE);
+  g_return_val_if_fail (key != NULL, FALSE);
+  g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
+  
+  g_return_val_if_fail (g_utf8_validate (val, -1, NULL), FALSE);
   
   gval = gconf_value_new(GCONF_VALUE_STRING);
 
@@ -2915,7 +3091,7 @@ gconf_engine_set_string  (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_bool    (GConfEngine* conf, const gchar* key,
-                   gboolean val, GError** err)
+                          gboolean val, GError** err)
 {
   GConfValue* gval;
 
@@ -2932,7 +3108,7 @@ gconf_engine_set_bool    (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_schema  (GConfEngine* conf, const gchar* key,
-                    const GConfSchema* val, GError** err)
+                          const GConfSchema* val, GError** err)
 {
   GConfValue* gval;
 
@@ -2950,11 +3126,12 @@ gconf_engine_set_schema  (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_list    (GConfEngine* conf, const gchar* key,
-                   GConfValueType list_type,
-                   GSList* list,
-                   GError** err)
+                          GConfValueType list_type,
+                          GSList* list,
+                          GError** err)
 {
   GConfValue* value_list;
+  GError *tmp_err = NULL;
   
   g_return_val_if_fail(conf != NULL, FALSE);
   g_return_val_if_fail(key != NULL, FALSE);
@@ -2963,7 +3140,13 @@ gconf_engine_set_list    (GConfEngine* conf, const gchar* key,
   g_return_val_if_fail(list_type != GCONF_VALUE_PAIR, FALSE);
   g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
 
-  value_list = gconf_value_list_from_primitive_list(list_type, list);
+  value_list = gconf_value_list_from_primitive_list(list_type, list, &tmp_err);
+
+  if (tmp_err)
+    {
+      g_propagate_error (err, tmp_err);
+      return FALSE;
+    }
   
   /* destroys the value_list */
   
@@ -2972,12 +3155,13 @@ gconf_engine_set_list    (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_pair    (GConfEngine* conf, const gchar* key,
-                   GConfValueType car_type, GConfValueType cdr_type,
-                   gconstpointer address_of_car,
-                   gconstpointer address_of_cdr,
-                   GError** err)
+                          GConfValueType car_type, GConfValueType cdr_type,
+                          gconstpointer address_of_car,
+                          gconstpointer address_of_cdr,
+                          GError** err)
 {
   GConfValue* pair;
+  GError *tmp_err = NULL;
   
   g_return_val_if_fail(conf != NULL, FALSE);
   g_return_val_if_fail(key != NULL, FALSE);
@@ -2993,7 +3177,14 @@ gconf_engine_set_pair    (GConfEngine* conf, const gchar* key,
   
 
   pair = gconf_value_pair_from_primitive_pair(car_type, cdr_type,
-                                              address_of_car, address_of_cdr);
+                                              address_of_car, address_of_cdr,
+                                              &tmp_err);
+
+  if (tmp_err)
+    {
+      g_propagate_error (err, tmp_err);
+      return FALSE;
+    }  
   
   return error_checked_set(conf, key, pair, err);
 }
@@ -3084,26 +3275,26 @@ gconf_handle_corba_exception(CORBA_Environment* ev, GError** err)
   switch (ev->_major)
     {
     case CORBA_NO_EXCEPTION:
-      CORBA_exception_free(ev);
+      CORBA_exception_free (ev);
       return FALSE;
       break;
     case CORBA_SYSTEM_EXCEPTION:
       if (err)
-        *err = gconf_error_new(GCONF_ERROR_NO_SERVER, _("CORBA error: %s"),
-                               CORBA_exception_id(ev));
-      CORBA_exception_free(ev);
+        *err = gconf_error_new (GCONF_ERROR_NO_SERVER, _("CORBA error: %s"),
+                                CORBA_exception_id (ev));
+      CORBA_exception_free (ev);
       return TRUE;
       break;
     case CORBA_USER_EXCEPTION:
-      {
+      {        
         ConfigException* ce;
 
-        ce = CORBA_exception_value(ev);
+        ce = CORBA_exception_value (ev);
 
         if (err)
-          *err = gconf_error_new(corba_errno_to_gconf_errno(ce->err_no),
-                                 ce->message);
-        CORBA_exception_free(ev);
+          *err = gconf_error_new (corba_errno_to_gconf_errno (ce->err_no),
+                                  ce->message);
+        CORBA_exception_free (ev);
         return TRUE;
       }
       break;
