@@ -535,6 +535,12 @@ main (int argc, char** argv)
       fprintf(stderr, _("You must specify a config source with --config-source when using --direct\n"));
       return 1;
     }
+
+  if (config_source && !use_local_source)
+    {
+      fprintf (stderr, _("You should use --direct when using a non-default configuration source\n"));
+      return 1;
+    }
   
   if (!gconf_init(argc, argv, &err))
     {
@@ -563,13 +569,20 @@ main (int argc, char** argv)
       else
         return 0;
     }
+
+  if (makefile_install_mode &&
+      g_getenv ("GCONF_DISABLE_MAKEFILE_SCHEMA_INSTALL"))
+    {
+      g_print (_("GCONF_DISABLE_MAKEFILE_SCHEMA_INSTALL is set, not installing schemas\n"));
+      makefile_install_mode = FALSE;
+    }
   
   if (makefile_install_mode)
     {
       g_assert (config_source == NULL);
 
       /* Try the environment variable */
-      config_source = g_getenv("GCONF_CONFIG_SOURCE");
+      config_source = g_getenv ("GCONF_CONFIG_SOURCE");
 
       if (config_source == NULL)
         {
@@ -649,9 +662,9 @@ main (int argc, char** argv)
   if (makefile_install_mode)
     {
       const gchar** args = poptGetArgs(ctx);
-      gint retval = do_makefile_install(conf, args);
-
-      gconf_engine_unref(conf);
+      gint retval = do_makefile_install (conf, args);
+      
+      gconf_engine_unref (conf);
 
       return retval;
     }
@@ -2284,12 +2297,15 @@ process_schema_list(GConfEngine* conf, xmlNodePtr node)
 
   while (iter != NULL)
     {
-      if (strcmp(iter->name, "schema") == 0)
-        process_schema(conf, iter);
-      else
-        fprintf(stderr, _("WARNING: node <%s> not understood below <schemalist>\n"),
-                iter->name);
-
+      if (iter->type == XML_ELEMENT_NODE)
+        {
+          if (strcmp(iter->name, "schema") == 0)
+            process_schema(conf, iter);
+          else
+            fprintf(stderr, _("WARNING: node <%s> not understood below <schemalist>\n"),
+                    iter->name);
+        }
+      
       iter = iter->next;
     }
 
