@@ -17,6 +17,33 @@
  * Boston, MA 02111-1307, USA.
  */
 
+
+
+
+
+
+
+
+
+
+
+/*
+ *
+ *
+ * DO NOT USE THESE CRAPPY TESTS AS EXAMPLE CODE. USE THE DOCS AND examples/*
+ *
+ *
+ *
+ */
+
+
+
+
+
+
+
+
+
 #include <gconf/gconf.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -844,6 +871,52 @@ check_list_storage(GConfEngine* conf)
   check_unset(conf);
 }
 
+static void
+check_utils (void)
+{
+  int i;
+  const char *escape_tests[] = {
+    "Hello",
+    "@@@@@@@@@@@@",
+    "foo\135\224\12\f\n\r@@\128@hello/foo",
+    "bar baz woo/",
+    "//////////",
+    "@128@@129@",
+    "/./././",
+    "",
+    "/",
+    "a",
+    "@"
+  };
+
+  i = 0;
+  while (i < G_N_ELEMENTS (escape_tests))
+    {
+      char *escaped;
+      char *unescaped;
+      char *whole_key;
+      
+      escaped = gconf_escape_key (escape_tests[i], -1);
+
+      /* escaped is a key element, not a key */
+      whole_key = g_strconcat ("/", escaped, NULL);
+      
+      check (gconf_valid_key (whole_key, NULL), "Escaped key '%s' is valid (original '%s')",
+             escaped, escape_tests[i]);
+      g_free (whole_key);
+      
+      unescaped = gconf_unescape_key (escaped, -1);
+      check (strcmp (escape_tests[i], unescaped) == 0,
+             "Unescaped key '%s' same as original '%s' (escaped was '%s'\n",
+             unescaped, escape_tests[i], escaped);
+
+      g_free (escaped);
+      g_free (unescaped);
+      
+      ++i;
+    }  
+}
+
 int 
 main (int argc, char** argv)
 {
@@ -859,6 +932,8 @@ main (int argc, char** argv)
       err = NULL;
       return 1;
     }
+
+  check_utils ();
   
   conf = gconf_engine_get_default();
 
